//$Id: canvasitem.cc,v 1.3 2001/06/09 13:23:20 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "canvasitem.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <gnome--/app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include <list>

#include "form-editor.h"

using namespace Guikachu::GUI::FormEditor;

CanvasItem::CanvasItem (Widgets::Widget  *widget_,
			FormEditor::Form *form_):
    widget (widget_),
    form (form_),
    selected (false)
{
    drag_context.dragging = false;
    drag_context.drag_occured = false;
    
    using namespace Gnome;
    
    list <UI::Info> popup_menu_list;
    /*
    popup_menu_list.push_back (
	UI::Item (_("Move"),
		  SigC::slot (this, &CanvasItem::move_cb)));
    */
    popup_menu_list.push_back (
	UI::Item (UI::Icon (GNOME_STOCK_PIXMAP_TRASH),
		  _("Remove"),
		  SigC::slot (this, &CanvasItem::remove_cb)));
    
    UI::fill (popup_menu, popup_menu_list, *(popup_menu.get_accel_group ()));

    widget->changed.connect (SigC::slot (this, &CanvasItem::update));
}

void CanvasItem::set_canvas_item (Gnome::CanvasItem *item_)
{
    item = item_;

    item->event.connect (SigC::slot (this, &CanvasItem::canvas_event_cb));
}

void CanvasItem::select (bool selected_)
{
}


void CanvasItem::remove_cb ()
{
    widget->get_form ()->remove_widget (widget);
}

void CanvasItem::move_cb ()
{
#if 0
    GdkEvent *event = gdk_event_get ();

    cerr << event->motion.x << endl << event->motion.y << endl;
    
    switch (event->type) {
    case GDK_ENTER_NOTIFY:
    case GDK_LEAVE_NOTIFY:
	item->get_canvas ()->window_to_world (event->crossing.x, event->crossing.y,
					      event->crossing.x, event->crossing.y);
	break;
	
    case GDK_MOTION_NOTIFY:
    case GDK_BUTTON_PRESS:
    case GDK_2BUTTON_PRESS:
    case GDK_3BUTTON_PRESS:
    case GDK_BUTTON_RELEASE:
	item->get_canvas ()->window_to_world (event->motion.x, event->motion.y,
					      event->motion.x, event->motion.y);
	break;
	
    default:
	break;
    }
    
    cerr << event->motion.x << endl << event->motion.y << endl;
    
    drag_begin (event);
#else
    GdkEvent event;

    int pointer_x, pointer_y;

    gdk_window_get_pointer (item->get_canvas ()->get_window (),
			    &pointer_x,
			    &pointer_y,
			    0);
    item->get_canvas ()->window_to_world (pointer_x, pointer_y,
					  event.button.x, event.button.y);
    event.button.time = GDK_CURRENT_TIME;
    
    drag_begin (&event);
#endif
}

int CanvasItem::canvas_event_cb (GdkEvent *e)
{
    /*
    if (drag_context.dragging)
	cerr << "canvas_event_cb" << endl;
    */
	
    // Begin drag
    if (e->type == GDK_BUTTON_PRESS &&
	e->button.button == 1 &&
	!drag_context.dragging)
	drag_begin (e);
    
    // Dragging
    if (e->type == GDK_MOTION_NOTIFY &&
	drag_context.dragging)
	drag_motion (e);

    // Ending drag
    if (e->type == GDK_BUTTON_RELEASE &&
	(e->button.button == 1) &&
	drag_context.dragging)
	drag_end (e);
    

    // Open property editor
    if ((e->type == GDK_BUTTON_PRESS) &&
	(e->button.button == 1))
	widget->show_editor ();
    

    // Context menu
    if ((e->type == GDK_BUTTON_PRESS) &&
	(e->button.button == 3))
    {
	popup_menu.popup (e->button.button, e->button.time);
    }
    
    return TRUE;
}

void CanvasItem::drag_begin (GdkEvent *e)
{
    drag_context.dragging = true;
    drag_context.drag_occured = false;

    drag_context.last_x = e->button.x;
    drag_context.last_y = e->button.y;
    
    drag_context.offset_x = drag_context.last_x - widget->get_x ();
    drag_context.offset_y = drag_context.last_y - widget->get_y ();
    
    Gdk_Cursor cursor (gdk_cursor_new (GDK_FLEUR));
    item->grab (GDK_POINTER_MOTION_MASK | GDK_BUTTON_MOTION_MASK |
		GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
		cursor, e->button.time);

//    cout << e->button.x << "\t" << e->button.y << endl;
}

void CanvasItem::drag_motion (GdkEvent *e)
{
    drag_context.drag_occured = true;
    
    double curr_x = e->motion.x;
    double curr_y = e->motion.y;
    
    // East bound
    if ((curr_x - drag_context.offset_x) > (GUIKACHU_SCREEN_WIDTH - 1))
	    curr_x = (GUIKACHU_SCREEN_WIDTH - 1) + drag_context.offset_x;
    
    // South bound
    if ((curr_y - drag_context.offset_y) > (GUIKACHU_SCREEN_HEIGHT - 1))
	curr_y = (GUIKACHU_SCREEN_HEIGHT - 1) + drag_context.offset_y;
    
    // West bound
    if ((curr_x - drag_context.offset_x) < 0)
	curr_x = drag_context.offset_x;
    
    // North bound
    if ((curr_y - drag_context.offset_y) < 0)
	curr_y = drag_context.offset_y;
    
    item->move (curr_x - drag_context.last_x,
		curr_y - drag_context.last_y);
    
    drag_context.last_x = curr_x;
    drag_context.last_y = curr_y;     
}

void CanvasItem::drag_end (GdkEvent *e)
{
    item->ungrab (e->button.time);
    drag_context.dragging = false;
    
    if (drag_context.drag_occured)
    {
	widget->set_x (drag_context.last_x - drag_context.offset_x);
	widget->set_y (drag_context.last_y - drag_context.offset_y);
    }
}
